/*******************************************************************************
 * Copyright (c) 2016 ALL4TEC & CEA LIST.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * Contributors:
 *     ALL4TEC & CEA LIST - initial API and implementation
 *******************************************************************************/
package org.polarsys.esf.fmea.profile.tools.util;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.transaction.RecordingCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.papyrus.uml.tools.utils.PackageUtil;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.util.UMLUtil;
import org.polarsys.esf.core.utils.ModelUtil;
import org.polarsys.esf.esfarchitectureconcepts.ISBlock;
import org.polarsys.esf.esffmea.ISBlockFMEA;
import org.polarsys.esf.esffmea.impl.ESFFMEAPackage;
import org.polarsys.esf.esffmea.impl.SBlockFMEA;
import org.polarsys.esf.fmea.profile.set.ESFFMEASet;

/**
 * Utilities for working with ESFFMEA.
 *
 * @author $Author: ogurcan $
 * @version $Revision: 168 $
 */
public final class ESFFMEAUtil {

    /** FMEA package name. */
    private static final String FMEA_PACKAGE_NAME = "FMEA"; //$NON-NLS-1$

    /** Apply ESFFMEA profile label. */
    private static final String APPLY_ESFFMEA_PROFILE_LABEL = "Apply ESFFMEA profile"; //$NON-NLS-1$

    /** Create Local Analysis package label. */
    private static final String CREATE_FMEA_PACKAGE_LABEL = "Create FMEA package"; //$NON-NLS-1$

    /** Create SBlock_LA label. */
    private static final String CREATE_SBLOCK_FMEA_LABEL = "Create SBlock_FMEA"; //$NON-NLS-1$

    /**
     * Default constructor.
     */
    private ESFFMEAUtil() {
    }

    /**
     * Create FMEA package.
     *
     * @param pSafetyArtifacts The SafetyArtifacts package
     */
    public static void createFMEAPackage(final Package pSafetyArtifacts) {
        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pSafetyArtifacts);

        RecordingCommand vCreateFMEAPackage =
            new RecordingCommand(vDomain, CREATE_FMEA_PACKAGE_LABEL) {

                @Override
                protected void doExecute() {
                    // Create the object 'Local Analysis' package
                    Package vFMEA = pSafetyArtifacts.createNestedPackage(FMEA_PACKAGE_NAME);
                    // Apply 'SFMEA' stereotype on 'FMEA' package
                    UMLUtil.StereotypeApplicationHelper.getInstance(vFMEA)
                        .applyStereotype(vFMEA, ESFFMEAPackage.eINSTANCE.getSFMEA());
                }
            };

        // Verify if command can be executed
        if (vCreateFMEAPackage.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vCreateFMEAPackage);
        }
    }

    /**
     * Create SBlockFMEA.
     *
     * @param pSFMEAPackage The FMEA package
     * @param pSBlock The SBlock to be local analysed
     */
    public static void createSBlockFMEA(final Package pSFMEAPackage, final ISBlock pSBlock) {
        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pSFMEAPackage);

        RecordingCommand vCreateSBlockLA = new RecordingCommand(vDomain, CREATE_SBLOCK_FMEA_LABEL) {

            @Override
            protected void doExecute() {
                // Create the object 'Class' of SBlock for FMEA
                Class vSBlockFMEAClass = pSFMEAPackage.createOwnedClass(pSBlock.getName(), true);
                // Apply 'SBlockFMEA' stereotype on 'SBlockFMEA'
                UMLUtil.StereotypeApplicationHelper.getInstance(vSBlockFMEAClass)
                    .applyStereotype(vSBlockFMEAClass, ESFFMEAPackage.eINSTANCE.getSBlockFMEA());

                // Link 'SBlockFMEA' stereotype to 'SBlock' stereotype
                ISBlockFMEA vSBlockFMEA =
                    (ISBlockFMEA) UMLUtil.getStereotypeApplication(vSBlockFMEAClass, SBlockFMEA.class);
                vSBlockFMEA.setSBlock(pSBlock);
            }
        };

        // Verify if command can be executed
        if (vCreateSBlockLA.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vCreateSBlockLA);
        }
    }

    /**
     * Apply ESFFMEA profile to the given model element.
     *
     * @param pModel The model target
     */
    public static void applyESFFMEAProfile(final Model pModel) {
        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pModel);

        RecordingCommand vApplyLAProfileCmd = new RecordingCommand(vDomain, APPLY_ESFFMEA_PROFILE_LABEL) {

            @Override
            protected void doExecute() {
                // Retrieve ESFFMEA profile and apply it
                Profile vESFFMEAProfile = (Profile) PackageUtil
                    .loadPackage(URI.createURI(ESFFMEASet.PROFILE_PATH), pModel.eResource().getResourceSet());

                if (vESFFMEAProfile != null) {
                    PackageUtil.applyProfile(pModel, vESFFMEAProfile, true);
                }
            }
        };

        // Verify if command can be executed
        if (vApplyLAProfileCmd.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vApplyLAProfileCmd);
        }
    }

}
