/*******************************************************************************
 * Copyright (c) 2016 ALL4TEC & CEA LIST.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     ALL4TEC & CEA LIST - initial API and implementation
 ******************************************************************************/
package org.polarsys.esf.localanalysis.profile.tools.util;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.transaction.RecordingCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.papyrus.uml.tools.utils.PackageUtil;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.util.UMLUtil;
import org.polarsys.esf.core.utils.ModelUtil;
import org.polarsys.esf.esfarchitectureconcepts.ISBlock;
import org.polarsys.esf.esfarchitectureconcepts.ISPort;
import org.polarsys.esf.esfarchitectureconcepts.SDirection;
import org.polarsys.esf.esflocalanalysis.ISBlockLAnalysis;
import org.polarsys.esf.esflocalanalysis.ISLocalAnalysis;
import org.polarsys.esf.esflocalanalysis.ISPortLAnalysis;
import org.polarsys.esf.esflocalanalysis.ISSystemEventsLibrary;
import org.polarsys.esf.esflocalanalysis.SDirectionLAnalysis;
import org.polarsys.esf.esflocalanalysis.impl.ESFLocalAnalysisPackage;
import org.polarsys.esf.esflocalanalysis.impl.SBlockLAnalysis;
import org.polarsys.esf.esflocalanalysis.impl.SPortLAnalysis;
import org.polarsys.esf.esflocalanalysis.impl.SSystemEventType;
import org.polarsys.esf.localanalysis.profile.set.ESFLocalAnalysisSet;

/**
 * Utilities for working with ESFLocalAnalysis.
 *
 * @author $Author: ymunoz $
 * @version $Revision: 168 $
 */
public final class ESFLocalAnalysisUtil {

    /** System Events Library name. */
    public static final String SYSTEM_EVENTS_LIBRARY_NAME = "SystemEventsLibrary"; //$NON-NLS-1$

    /** Create System Events Library label. */
    public static final String CREATE_SYSTEM_EVENTS_LIBRARY_LABEL = "Create SystemEventsLibrary"; //$NON-NLS-1$

    /** Create SSystemEventType label. */
    public static final String CREATE_SYSTEM_EVENT_TYPE_LABEL = "Create SSystemEventType"; //$NON-NLS-1$

    /** LocalAnalysis package name. */
    public static final String LOCAL_ANALYSIS_PACKAGE_NAME = "LocalAnalysis"; //$NON-NLS-1$

    /** Apply ESFLocalAnalysis profile label. */
    public static final String APPLY_ESFLOCALANALYSIS_PROFILE_LABEL = "Apply ESFLocalAnalysis profile"; //$NON-NLS-1$

    /** Create Local Analysis package label. */
    public static final String CREATE_LOCALANALYSIS_PACKAGE_LABEL = "Create LocalAnalysis package"; //$NON-NLS-1$

    /** Create SBlock_LA label. */
    public static final String CREATE_SBLOCK_LA_LABEL = "Create SBlock_LA"; //$NON-NLS-1$

    /**
     * Default constructor.
     */
    private ESFLocalAnalysisUtil() {
    }

    /**
     * Create the System Events Library.
     *
     * @param pLocalAnalysis The LocalAnalysis package where the library must be created
     */
    public static void createSSystemEventsLibrary(final Package pLocalAnalysis) {
        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pLocalAnalysis);

        RecordingCommand vCreateSystemEventsLibrary =
            new RecordingCommand(vDomain, CREATE_SYSTEM_EVENTS_LIBRARY_LABEL) {

                @Override
                protected void doExecute() {
                    // Create the object 'SystemEventsLibrary' package
                    Package vSSystemEventsLibrary = pLocalAnalysis.createNestedPackage(SYSTEM_EVENTS_LIBRARY_NAME);
                    // Apply 'SSystemEventsLibrary' stereotype on 'SystemEventsLibrary' package
                    UMLUtil.StereotypeApplicationHelper.getInstance(vSSystemEventsLibrary).applyStereotype(
                        vSSystemEventsLibrary,
                        ESFLocalAnalysisPackage.eINSTANCE.getSSystemEventsLibrary());
                }
            };

        // Verify if command can be executed
        if (vCreateSystemEventsLibrary.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vCreateSystemEventsLibrary);
        }
    }

    /**
     * Create a new SSystemEventType element.
     *
     * @param pSSystemEventsLibrary The SSystemEventsLibrary package where the SystemEventTypes must be created
     */
    public static void createSSystemEventType(final Package pSSystemEventsLibrary) {

        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pSSystemEventsLibrary);

        RecordingCommand vCreateSystemEventType = new RecordingCommand(vDomain, CREATE_SYSTEM_EVENT_TYPE_LABEL) {

            @Override
            protected void doExecute() {
                String vName = SSystemEventType.class.getSimpleName() + pSSystemEventsLibrary.getOwnedElements().size();
                // Create a new SSystemEventType element
                Class vSSystemEventType = pSSystemEventsLibrary.createOwnedClass(vName, false);
                // Apply 'SSystemEventType' stereotype on the created class
                UMLUtil.StereotypeApplicationHelper.getInstance(vSSystemEventType)
                    .applyStereotype(vSSystemEventType, ESFLocalAnalysisPackage.eINSTANCE.getSSystemEventType());
            }
        };

        // Verify if command can be executed
        if (vCreateSystemEventType.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vCreateSystemEventType);
        }
    }

    /**
     * Create LocalAnalysis package.
     *
     * @param pSafetyArtifacts The SafetyArtifacts package
     */
    public static void createLocalAnalysisPackage(final Package pSafetyArtifacts) {
        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pSafetyArtifacts);

        RecordingCommand vCreateLocalAnalysisPackage =
            new RecordingCommand(vDomain, CREATE_LOCALANALYSIS_PACKAGE_LABEL) {

                @Override
                protected void doExecute() {
                    // Create the object 'Local Analysis' package
                    Package vLocalAnalysis = pSafetyArtifacts.createNestedPackage(LOCAL_ANALYSIS_PACKAGE_NAME);
                    // Apply 'SLocalAnalysis' stereotype on 'LocalAnalysis' package
                    UMLUtil.StereotypeApplicationHelper.getInstance(vLocalAnalysis)
                        .applyStereotype(vLocalAnalysis, ESFLocalAnalysisPackage.eINSTANCE.getSLocalAnalysis());
                }
            };

        // Verify if command can be executed
        if (vCreateLocalAnalysisPackage.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vCreateLocalAnalysisPackage);
        }
    }

    /**
     * Create SBlockLAnalysis.
     *
     * @param pSLocalAnalysisPackage The LocalAnalysis package
     * @param pSBlock The SBlock to be local analysed
     */
    public static void createSBlockLAnalysis(final Package pSLocalAnalysisPackage, final ISBlock pSBlock) {
        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pSLocalAnalysisPackage);

        RecordingCommand vCreateSBlockLA = new RecordingCommand(vDomain, CREATE_SBLOCK_LA_LABEL) {

            @Override
            protected void doExecute() {
                // Create the object 'Class' of SBlock for Local Analysis
                Class vSBlockLA = pSLocalAnalysisPackage.createOwnedClass(pSBlock.getName(), true);
                // Apply 'SBlockLAnalysis' stereotype on 'SBlockLA'
                UMLUtil.StereotypeApplicationHelper.getInstance(vSBlockLA)
                    .applyStereotype(vSBlockLA, ESFLocalAnalysisPackage.eINSTANCE.getSBlockLAnalysis());

                // Link 'SBlockLAnalysis' stereotype to 'SBlock' stereotype
                ISBlockLAnalysis vSBlockLAnalysis =
                    (ISBlockLAnalysis) UMLUtil.getStereotypeApplication(vSBlockLA, SBlockLAnalysis.class);
                vSBlockLAnalysis.setSBlock(pSBlock);

                // Create the ports for SBlockLA
                for (ISPort vSPort : pSBlock.getOwnedSPortsList()) {
                    Port vPortLA = vSBlockLA.createOwnedPort(vSPort.getName(), null);
                    // Apply 'SPortLAnalysis' stereotype on 'SBlockLA'
                    UMLUtil.StereotypeApplicationHelper.getInstance(vPortLA)
                        .applyStereotype(vPortLA, ESFLocalAnalysisPackage.eINSTANCE.getSPortLAnalysis());

                    // Link 'SPortLAnalysis' stereotype to 'SPort' stereotype
                    ISPortLAnalysis vSPortLAnalysis =
                        (ISPortLAnalysis) UMLUtil.getStereotypeApplication(vPortLA, SPortLAnalysis.class);
                    vSPortLAnalysis.setSPort(vSPort);
                    SDirection vSDirection = vSPort.getSDirection();
                    vSPortLAnalysis.setSDirectionLAnalysis(SDirectionLAnalysis.get(vSDirection.getValue()));

                }
            }
        };

        // Verify if command can be executed
        if (vCreateSBlockLA.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vCreateSBlockLA);
        }
    }

    /**
     * Apply ESFLocalAnalysis profile to the given model element.
     *
     * @param pModel The model target
     */
    public static void applyESFLocalAnalysisProfile(final Model pModel) {
        TransactionalEditingDomain vDomain = ModelUtil.getTransactionalEditingDomain(pModel);

        RecordingCommand vApplyLAProfileCmd = new RecordingCommand(vDomain, APPLY_ESFLOCALANALYSIS_PROFILE_LABEL) {

            @Override
            protected void doExecute() {
                // Retrieve ESFLocalAnalysis profile and apply it
                Profile vESFLocalAnalysisProfile = (Profile) PackageUtil
                    .loadPackage(URI.createURI(ESFLocalAnalysisSet.PROFILE_PATH), pModel.eResource().getResourceSet());

                if (vESFLocalAnalysisProfile != null) {
                    PackageUtil.applyProfile(pModel, vESFLocalAnalysisProfile, true);
                }
            }
        };

        // Verify if command can be executed
        if (vApplyLAProfileCmd.canExecute()) {
            // Execute command
            vDomain.getCommandStack().execute(vApplyLAProfileCmd);
        }
    }

    /**
     * Retrieve 'SystemEventsLibrary' package.
     *
     * @param pLocalAnalysisPackage The 'LocalAnalysis' package
     * @return The 'SystemEventsLibrary' package
     */
    public static Package getSystemEventsLibraryPackage(final Package pLocalAnalysisPackage) {
        Package vSystemEventsLibrary = null;
        Boolean vFound = false;
        List<Package> vPackagesList = pLocalAnalysisPackage.getNestedPackages();
        Iterator<Package> vIterator = vPackagesList.iterator();

        while (vIterator.hasNext() && !vFound) {
            Package vPackage = vIterator.next();
            if (UMLUtil.getStereotypeApplication(vPackage, ISSystemEventsLibrary.class) != null) {
                vFound = true;
                vSystemEventsLibrary = vPackage;
            }
        }

        return vSystemEventsLibrary;
    }

    /**
     * Get LocalAnalysis package.
     *
     * @param pSafetyArtifacts The SafetyArtifacts package
     * @return The LocalAnalysis package
     */
    public static Package getLocalAnalysisPackage(final Package pSafetyArtifacts) {
        Package vLocalAnalysisPackage = null;
        Boolean vFound = false;
        List<Package> vPackagesList = pSafetyArtifacts.getNestedPackages();
        Iterator<Package> vIterator = vPackagesList.iterator();

        while (vIterator.hasNext() && !vFound) {
            Package vPackage = vIterator.next();
            if (UMLUtil.getStereotypeApplication(vPackage, ISLocalAnalysis.class) != null) {
                vFound = true;
                vLocalAnalysisPackage = vPackage;
            }
        }

        return vLocalAnalysisPackage;
    }

    /**
     * Get the LocalAnalysis of a SBlock (SBlockLAnalysis).
     *
     * @param pLocalAnalysisPackage The LocalAnalysis package
     * @param pSBlock The SBlock which is analysed by SBlockLAnalysis
     * @return The SBlockLAnalysis
     */
    public static ISBlockLAnalysis getSBlockLAnalysisBySBlock(
        final Package pLocalAnalysisPackage,
        final ISBlock pSBlock) {
        ISBlockLAnalysis vSBlockLAnalysis = null;
        List<Element> vElementsList = pLocalAnalysisPackage.getOwnedElements();
        Iterator<Element> vIterator = vElementsList.iterator();

        while (vSBlockLAnalysis == null && vIterator.hasNext()) {
            Element vElement = vIterator.next();

            if (vElement instanceof Class) {
                if (UMLUtil.getStereotypeApplication(vElement, SBlockLAnalysis.class) != null) {
                    vSBlockLAnalysis =
                        (ISBlockLAnalysis) UMLUtil.getStereotypeApplication(vElement, ISBlockLAnalysis.class);
                    if (!vSBlockLAnalysis.getSBlock().equals(pSBlock)) {
                        vSBlockLAnalysis = null;
                    }
                }
            }
        }
        return vSBlockLAnalysis;
    }
}
